/*******************************************************************************/
/*** This module was written 22. DEZ 1993 by Katzlberger Thomas, Innsbruck,  ***/
/*** AUSTRIA and is Part of the distribution of the Super Solitaire App, as  ***/
/*** an example module.                                                      ***/
/***                                                                         ***/
/*** YOU MAY:   - reuse this code for writing Super Solitaire Bundles.       ***/
/***            - modify the module as long as you rename the Class and      ***/
/***              replace the author's message by your own.                  ***/
/***            - reuse this code for any Public Domain, especially not      ***/
/***              commercial offer.                                          ***/
/***                                                                         ***/
/*** =========>>> Best results setting TAB-Size to 4 characters <<<========= ***/
/***                                                                         ***/
/*******************************************************************************/

/*******************************   INTERFACES   ********************************/

enum PokerPicture {Mix,Pair,TwoPair,ThreeOAK,Flush,Straight,FullHouse,FourOAK,RoyalFlush};
enum PokerTactic  {Honest,LowHigh,Equal,HighLow,LittleBluff,TheBigBluff};

#import "Poker.h"
#import "ReusableObjects/Random.h"

	@interface Poker(Bundle)
	
	- loadBundle:sender;
	
	@end
	
	@interface Poker(Private)
	
	- setupViews;
	- (void) clearAll;
	- dealOut;
	
	@end
	
	@interface Poker(Window)
	@end
	
	@interface Poker(GameLogic)
	
	- evaluateHand:hand;

	@end

/*******************************     BUNDLE     ********************************/
	@implementation Poker(Bundle)
	
	- loadBundle:bundle
	{		
		char p[1025];

		[[NXHelpPanel new] addSupplement:"Help" inPath:[bundle directory]];
		[bundle getPath:p forResource:[[self class] name] ofType:"nib"];		
		[NXApp loadNibFile:p owner:self];
		
		return self;
	}
		
	@end
	
/*******************************     WINDOW     ********************************/
	@implementation Poker(Window)
	
	- windowWillClose:sender
	{
		[NXApp delayedFree:self];
		return self;
	}
		
	@end
	
/*******************************     PRIVATE    ********************************/
@implementation Poker(Private)

- setupViews
{
	[toTalon enableDrag:NO];
	[toBlock enableDrag:NO];
	[toBlock grow:GROW_UP];

	[toHand1 setDeltaX:CARD_WIDTH+1];
	[toHand2 setDeltaX:CARD_WIDTH+1];

	[toHand1 enableDrop:NO];

	[toHand1 enableDrag:NO];
	[toHand2 enableDrag:NO];

	return self;
}

- (void) clearAll
{
	[toHand1 clear];
	[toHand2 clear];
	
	[toTalon clear];
	[toBlock clear];
}

- dealOut
{
	return self;
}

@end

@implementation Poker

- initBundle:bundle
{
	[super init];
	
	#include "../Demo.h"

	[self loadBundle:bundle];
	[self setupViews];

	rule = 0; 
	rand = [[Random alloc] init];
	
	[self newGame:self];

	[toMyWindow makeKeyAndOrderFront:self];

	return self;
}

- free
{
	return [super free];
}

- deal:sender
{
	if([toTalon count]<21) [self newGame:self];
	rienNeVaPlus = NO;

	[toHand1 clear];
	[toHand2 clear];

	[toScoreForm setStringValue:"" at:0];
	[toScoreForm setStringValue:"" at:1];

	int i;
	for(i=0;i<3;i++)
	{	[[toTalon lastCard] performDragToView:toHand1];
		[toHand1 addCard:[toTalon removeCard]];
	}		
	for(i=0;i<3;i++)
	{	[[toTalon lastCard] performDragToView:toHand2];
		[toHand2 objectDropped:[toTalon lastCard]];
	}
	for(i=0;i<2;i++)
	{	[[toTalon lastCard] performDragToView:toHand1];
		[toHand1 addCard:[toTalon removeCard]];
	}		
	for(i=0;i<2;i++)
	{	[[toTalon lastCard] performDragToView:toHand2];
		[toHand2 objectDropped:[toTalon lastCard]];
	}

	[toHand2 flipCards:YES];
	[toHand2 sort:INC_ACE_HIGH];
	[toHand2 reset];

	return self;
}

- (int)play 		// the brain ..... 
{
	id l=[toHand1 list];
	int val[6],kin[6],good[6],i;
	
	[toHand1 sort:INC_ACE_HIGH];
	
	for(i=0;i<5;i++)
		good[i]=NO;
		
	for(i=0;i<5;i++)
	{	val[i] = [[l objectAt:i] value];
		if(val[i]==CARD_ACE) val[i]+=CARD_KING;
		kin[i] = [[l objectAt:i] kind ];
	}
	
	for(i=0;i<4;i++)				// Straight
	{	if(val[i]+1 != val[i+1]) break;
		if(i==4) return 0;
	}
	
	for(i=0;i<4;i++)				// Flush
	{	if(kin[i] != kin[i+1]) break;
		if(i==4) return 0;
	}
	
	int cnt=0;
	for(i=0;i<4;i++)				// Pairs,3oak,4oak,FullHouse
		if(val[i] == val[i+1])
		{	good[i]=YES;
			good[i+1]=YES;
			cnt++;
		}
	
	if(cnt==0)
	for(i=0;i<5;i++)				// try for high Pair
		if(val[i]>10) 
		{	good[i]=([rand percent]>0.1);
			cnt++;
		}

	// The Big Bluff
	if(cnt==0)
	if([rand percent]<0.33)
	{	for(i=0;i<5;i++)
			good[i]=([rand percent]>0.5);
	}

	for(i=0;i<5;i++)
		if(good[i]==NO)
			[[l objectAt:i] setHighlighted:YES];

	cnt = [toHand1 sendSelectedCardsTo:toBlock];

	for(i=0;i<cnt;i++)
	{	[[toTalon lastCard] performDragToView:toHand1];
		[toHand1 addCard:[toTalon removeCard]];
	}

	[toHand1 sort:INC_ACE_HIGH];
	[toHand1 reset];

	return cnt;
}

- nextCard:sender
{
	int i,cnt;
	char str[100];

	if(rienNeVaPlus == YES) return self;
	rienNeVaPlus = YES;

	[sender setEnabled:NO];
	[sender perform:@selector(setEnabled:) with:(id)YES afterDelay:200 cancelPrevious:YES];

	cnt = [toHand2 sendSelectedCardsTo:toBlock];
	
	for(i=0;i<cnt;i++)
	{	[[toTalon lastCard] performDragToView:toHand2];
		[toHand2 objectDropped:[[toTalon lastCard] showFace:YES]];
	}

	[toHand2 sort:INC_ACE_HIGH];
	[toHand2 reset];

	sprintf(str,"Changed %i Cards.",cnt);
	[toScoreForm setStringValue:str at:1];
	
	cnt = [self play];
	
	sprintf(str,"Changed %i Cards.",cnt);
	[toScoreForm setStringValue:str at:0];

	return self;
}

- stand:sender
{	
	int extr,hand;
	
	if(rienNeVaPlus == NO) return self;
	
	[toHand1 flipCards:YES];

	[self evaluateHand:toHand1];
	[toScoreForm setStringValue:handString at:0];
	hand=handPoints;
	extr=extraPoints;
	[self evaluateHand:toHand2];
	[toScoreForm setStringValue:handString at:1];
	
	BOOL lost = NO;
	
	rienNeVaPlus = NO;

	if(hand>handPoints) lost = YES;
	
	if(hand==handPoints)
	{	if(extr>extraPoints) lost = YES;
		else
			if(extr==extraPoints) return self;
	}
	
	if(lost)
		[toAccount setIntValue:[toAccount intValueAt:0]+[toPot intValue] at:0];
	else
		[toAccount setIntValue:[toAccount intValueAt:1]+[toPot intValue] at:1];
	
	[toPot setIntValue:10];
		
	return self;
}

- newGame:sender
{	
	// Talon Default: no drop  , draggable
	// Block Default: droppable, draggable

	[self clearAll];

	[toTalon newDeck];
	[toTalon shuffle];
	[toTalon reset];

	flips = 1;
	char  str[20];
	sprintf(str,"Flips: %i",flips);
	[toFlipButton setEnabled:YES];
	[toFlipButton setTitle:str];
	
	rienNeVaPlus = NO;

	if([toAccount intValueAt:1]<=0)
		[toAccount setIntValue:500 at:1];
		
    return self;
}

- flipPile:sender
{
	return self;
}

- right:sender
{
	if(rienNeVaPlus) return self;
	if([toMoney intValue]>=[toAccount intValueAt:1]) return self;

	[toMoney setIntValue:[toMoney intValue]+1];
	return self;
}

- left:sender
{
	if(rienNeVaPlus) return self;

	if([toMoney intValue]<1) return self;
	[toMoney setIntValue:[toMoney intValue]-1];
	return self;
}

- toggleRule:sender
{
	if(NX_ALERTDEFAULT!=NXRunAlertPanel(0,"This will redeal !\nAre you sure ?","YES","NO",0))
		return [sender setState:![sender state]];
		
	if(rule==0) rule=1; else rule=0;
	[self newGame:self];
	
	return self;
}

@end

/********************************   GAMELOGIC  ********************************/
@implementation Poker(GameLogic)

- blockStack:stack cardDropped:card
{
	[card showFace:NO];
	return self;
}

- evaluateHand:hand
{
	id l=[hand list];
	int val[6],kin[6],i;
	
	extraPoints=0;
	
	[hand sort:INC_ACE_HIGH];
			
	extraPoints=0;
	for(i=0;i<5;i++)
	{	val[i] = [[l objectAt:i] value];
		if(val[i]==CARD_ACE) val[i]+=CARD_KING;
		kin[i] = [[l objectAt:i] kind];
		extraPoints += val[i];
	}
	
	val[5]=0;
	
	for(i=0;i<4;i++)					// RoyalFlush
	{	if(val[i]+1 != val[i+1]) break;
		if(kin[i] != kin[i+1]) break;
		if(i==3)
		{	handPoints = RoyalFlush;
			handString = "RoyalFlush";
			return self;
		}
	}
	
	for(i=0;i<4;i++)					// Straight 45678
	{	if(val[i]+1 != val[i+1]) break;
		if(i==3)
		{	handPoints = Straight;
			handString = "Straight";
			return self;
		}
	}
	
	for(i=0;i<4;i++)					// Flush same color
	{	if(kin[i] != kin[i+1]) break;
		if(i==3)
		{	handPoints = Flush;
			handString = "Flush";
			return self;
		}		
	}
	
	int n=0,p=0,f[3] = {0,0,0};
	
	for(i=0;i<4;i++)					// Pairs,3oak,4oak,FullHouse
	{	if(val[i] == val[i+1]) 
			p++;
		else
		{	if(p>0) f[n++]=p;
			p=0;
		}
	}
		
	if(p>0) f[n++]=p;

	if(f[0]==3)
	{	handPoints = FourOAK;
		handString = "Four of a kind";
		return self;
	}
	
	if(((f[0]==2) && (f[1]==1))	|| ((f[0]==1) && (f[1]==2)))
	{	handPoints = FullHouse;
		handString = "Full House";
		return self;
	}
	
	if((f[0]==1) && (f[1]==1))	
	{	handPoints = TwoPair;
		handString = "Two Pair";
		return self;
	}
	
	if(f[0]==2)
	{	handPoints = ThreeOAK;
		handString = "Three of a kind";
	}

	if(f[0]==1)
	{	handPoints = Pair;
		handString = "Pair";
	}

	if(f[0]==0)
	{	handPoints = Mix;
		handString = "Tutti frutti";
	}
	
	return self;
}

@end

