/*******************************   INTERFACES   ********************************/

#import "Solitaire.h"

	@interface Solitaire(Bundle)
	
	- loadBundle:sender;
	
	@end
	
	@interface Solitaire(Private)
	
	- (void) setLogicEvaluators:(BOOL)flag;
	- setupViews;
	- (void) clearAll;
	- dealOut;
	
	@end
	
	@interface Solitaire(Window)
	@end
	
	@interface Solitaire(GameLogic)
	
	- hintUsed:sender;
	
	@end

/*******************************     BUNDLE     ********************************/
	@implementation Solitaire(Bundle)
	
	- loadBundle:bundle
	{		
		char p[1025];

		[[NXHelpPanel new] addSupplement:"Help" inPath:[bundle directory]];
		[bundle getPath:p forResource:[[self class] name] ofType:"nib"];		
		[NXApp loadNibFile:p owner:self];
		
		return self;
	}
		
	@end
	
/*******************************     WINDOW     ********************************/
	@implementation Solitaire(Window)
	
	- windowWillClose:sender
	{
		[[MainDelegate new] collect:nil];
		[NXApp delayedFree:self];
		return self;
	}
	
	- windowDidBecomeKey:sender
	{
		[self  setLogicEvaluators:YES];
		return self;
	}
	
	- windowDidResignKey:sender
	{
		[self  setLogicEvaluators:NO];
		return self;
	}
	
	@end
	
/*******************************     PRIVATE    ********************************/
@implementation Solitaire(Private)

- (void) setLogicEvaluators:(BOOL)flag
{
	if(flag)
	{	[PlayStack  setLogicEvaluator:self];
		[TalonStack setLogicEvaluator:self];
		[BlockStack setLogicEvaluator:self];
		[DropStack setLogicEvaluator:self];
	}
	else
	{	[PlayStack  setLogicEvaluator:nil];
		[TalonStack setLogicEvaluator:nil];
		[BlockStack setLogicEvaluator:nil];
		[DropStack setLogicEvaluator:nil];
	}
}

- setupViews
{
	return self;
}

- (void) clearAll
{
	[toDropper1 clear];
	[toDropper2 clear];
	[toDropper3 clear];
	[toDropper4 clear];

	[toStack1 	clear];
	[toStack2 	clear];
	[toStack3 	clear];
	[toStack4 	clear];
	[toStack5 	clear];
	[toStack6 	clear];
	[toStack7 	clear];
	
	[toTalon clear];
	[toBlock clear];
}

- dealOut
{
	int i;

	for(i=0;i<6;i++)
		[toStack1 addCard:[toTalon removeCard]];
	
	for(i=0;i<5;i++)
		[toStack2 addCard:[toTalon removeCard]];

	for(i=0;i<4;i++)
		[toStack3 addCard:[toTalon removeCard]];

	for(i=0;i<3;i++)
		[toStack4 addCard:[toTalon removeCard]];

	[toStack5 addCard:[toTalon removeCard]];
	[toStack5 addCard:[toTalon removeCard]];

	[toStack6 addCard:[toTalon removeCard]];
	
	[self deal:self];

	return self;
}

@end

@implementation Solitaire

- initBundle:bundle
{
	[super init];
	
	#include "../Demo.h"

	[self loadBundle:bundle];
	[self setupViews];

	rule = 0; 

	[self setLogicEvaluators:YES];
	[self newGame:self];

	[toMyWindow makeKeyAndOrderFront:self];

	return self;
}

- free
{
	return [super free];
}

- deal:sender
{
	[[[toTalon lastCard] showFace:YES] performDragToView:toStack1];
	[[[toTalon lastCard] showFace:YES] performDragToView:toStack2];
	[[[toTalon lastCard] showFace:YES] performDragToView:toStack3];
	[[[toTalon lastCard] showFace:YES] performDragToView:toStack4];
	[[[toTalon lastCard] showFace:YES] performDragToView:toStack5];
	[[[toTalon lastCard] showFace:YES] performDragToView:toStack6];
	[[[toTalon lastCard] showFace:YES] performDragToView:toStack7];

	return self;
}

- newGame:sender
{	
	[[MainDelegate new] collect:self];

	[self clearAll];

	[[toTalon newDeck] shuffle];
	cardsOut = 52;
	
	hints = 0;
	[self hintUsed:self];
	flips = 9;
	
	char  str[20];
	sprintf(str,"Flips: %i",flips);
	[toFlipButton setEnabled:YES];
	[toFlipButton setTitle:str];

	[self dealOut];

	
    return self;
}

- flipPile:sender	// do not use sender (may be MainDelegate)
{
	if([toTalon isEmpty] == NO)	return self;

	if(flips<=0) return self;
	flips--;
	if(flips<=0) [toFlipButton setEnabled:NO];
	char str[20];
	sprintf(str,"Flips: %i",flips);
	[toFlipButton setTitle:str];

	id card;
	
	while(card = [toBlock removeCard])
		[toTalon addCard:card];
		
	return self;
}

- nextCard:sender
{
	[sender setEnabled:NO];
	[sender perform:@selector(setEnabled:) with:(id)YES afterDelay:200 cancelPrevious:YES];

	[[toTalon lastCard] performDragToView:toBlock];

	return self;
}

- toggleRule:sender
{
	if(NX_ALERTDEFAULT!=NXRunAlertPanel(0,"This will redeal !\nAre you sure ?","YES","NO",0))
		return [sender setState:![sender state]];
		
	if(rule==0) rule=1; else rule=0;
	[self newGame:self];
	
	return self;
}

@end

/********************************   GAMELOGIC  ********************************/
@implementation Solitaire(GameLogic)

/******************************************************************************/
/*** you have to comment these two functions out if you do not use them !!! ***/
/******************************************************************************/

/*
- (BOOL) dropStack:stack acceptsCard:upperCard onTopOf:lowerCard
{
	return NO;
}
*/

- dropStack:stack cardDropped:card
{
	cardsOut --;
	
	if(cardsOut==0)
		[[MainDelegate new] gameWon];
	
	return self;
}

- dropStack:stack cardRemoved:card
{
	cardsOut ++;
	return self;
}	

- (BOOL) playStack:stack acceptsCard:upperCard onTopOf:lowerCard
{
	if([upperCard superview] == toTalon) return NO;

	if(lowerCard == nil)
	{	if([upperCard value] == CARD_KING)
			return YES;
		else
			return NO;
	}
		
	if([upperCard isRed] == [lowerCard isRed]) return NO;	
	if([upperCard value] != ([lowerCard value]-1)) return NO;
	
	return YES;
}

- playStack:playStack cardDropped:card
{
	return self;
}

- blockStack:stack cardDropped:card
{
	return self;
}

- talonStack:stack cardRemoved:card		// card has no superview at this point !!
{
	// [card showFace:YES];
	return self;
}

- (int)hints
{
	return hints;
}

- hintUsed:sender
{
	static char title[1024] = "";
	char help[1024];
	
	if(title[0]==0) strcpy(title,[toMyWindow title]);
	
	if(hints==0)
		strcpy(help,title);
	else
	{	sprintf(help,"%s Hints: %i",title,hints);
		cardsOut++; // this disables the Bonus
	}
	
	[toMyWindow setTitle:help];
	hints++;
	
	return self;
}

@end

