// RODatabaseManager.m 19.1.93

#ifndef __RODBManager__
#define __RODBManager__

#ifdef __GNUG__
extern "Objective-C" {
#endif

#import	<objc/Object.h>
#import	<store/IXStoreFile.h>
#import	<indexing/IXPostingList.h>
#import <indexing/IXRecordManager.h>

#ifdef __GNUG__
}
#endif

typedef unsigned int handle;  // == handle (IX Kit)

@protocol RODBMAttributes

+ addAttributesTo:recordManager;
+ (const char*)storeName;  			// informal; takes [recordClass name] if not responding.

@end

@interface RODatabaseManager:IXRecordManager
{
    id 			recordClass;

	IXStore*	storeFile;
	handle		refHandle;
	
	char*     errstr;

	struct _dbm_flags
	{
		#ifdef __BIG_ENDIAN__
        unsigned int alertPanels:1;
		unsigned int autoTransact:1;
		unsigned int useDefaultsDatabase:1;
		
		unsigned int reserved:29;
		#else
		unsigned int reserved:29;
		
		unsigned int useDefaultsDatabase:1;
		unsigned int autoTransact:1;
        unsigned int alertPanels:1;
		#endif
	} dbm_flags;
}

- initClass:(id <RODBMAttributes>)c forWriting:(BOOL)wr;
	// Needs: registered DefaultValue e.g.: 
	//        { "NameOfRecordClass.DBPath","~/Library/Store/myfile.store" }
	//		  StoreName will be set to "NameOfRecordClass"

- initClass:(id <RODBMAttributes>)c inStore:(const char *)s inFile:(const char *)f 
	forWriting:(BOOL)wr;

- enableAlertPanels:(BOOL)flag;
- saveStorePathToDefaultsDatabase:(BOOL)flag;	// defaults to YES
- (void) disableAutotransaction;
- (void) enableAutotransaction;

- (int) errorNumber;
- (const char*) errorString;

@end

@interface RODatabaseManager(Querying)

// Don't forget to free these Lists (and Objects) ! (only TableController does it for you)

- (IXPostingList*) selectAllFromAttribute:(const char*) attr;

- (IXPostingList*) query:(const char*)theExpression 
  forAttribute:(const char*)attr andIntersect:(IXPostingList*)l;
- (handle) select:(const char*)value fromAttribute:(const char*)attr;

- (IXPostingList*) createPostingList;	// returns a empty posting list (self as datasource)

@end

@interface RODatabaseManager(Database)

- (id)load:(handle)h;
- (handle) insert:object;
- (void) replace:(handle)h with:object;
- (void) delete:(handle)h;

@end


#endif
