// RODraggableImageView.h 13.12.1993

#import <ReusableObjects/RODraggableView.h>
#import <ReusableObjects/ROImage.h>

#define DDB_DIV_DISSOLVING_STEPS "RODraggableImageView.DissolvingSteps"

@interface RODraggableImageView:RODraggableView
{
	ROImage	*myImage;
	ROImage *altImage;

	int state;	// 1 == alternate
	
	struct _DIVFlags
	{	
		#ifdef __BIG_ENDIAN__
		unsigned int highlightFlag:1;
		unsigned int highlightClick:1;
		unsigned int mouseDown:1;
		unsigned int dissolve:1;
		
		unsigned int disCount:8;
		unsigned int imageSized:1;
		unsigned int reserved:23;
		#else
		unsigned int reserved:23;
		unsigned int imageSized:1;
		unsigned int disCount:8;
		
		unsigned int dissolve:1;
		unsigned int mouseDown:1;
		unsigned int highlightClick:1;
		unsigned int highlightFlag:1;
		#endif
	} divFlags;
	
	float halfDiagonal;
	NXPoint comopsiteOrigin;
	NXPoint point;
}

+ setDrawWhenRotated:(BOOL)b;
+ (BOOL)doesDrawWhenRotated;
+ (void)sync;
+ (void)syncWithTimeout:(int)seconds;
+ setInitOpaque:(BOOL)b;		// default YES
+ setInitClipping:(BOOL)b;		// default YES
+ setDissolvingSteps:(int)d;
+ (int)dissolvingSteps;
+ defaultsChanged:sender;

- sizeImageCacheForRotation;
- (NXRect *)calcFrame;

- initWithIcon:(const char*)name alternateIcon:(const char*)aname;

- setState:(int)s;
- (int)state;

- setHighlighted:(BOOL)h;
- (BOOL)isHighlighted;
- toggleHighlight;
- setHighlightWhenClicked:(BOOL)f;
- (BOOL)highlightWhenClicked;

- (ROImage*)setIcon:(const char*)name;
- (ROImage*)setImage:(ROImage*)img;
- (ROImage*)image;

- (ROImage*)setAlternateIcon:(const char*)name;
- (ROImage*)setAlternateImage:(ROImage*)img;
- (ROImage*)alternateImage;

- eraseWithGray:(float)gray;

- dissolveToVisible:sender;
- dissolveToInvisible:sender;

- dissolve:(BOOL)f;				// uses animator of superclass
- setDissolve:(float)f;
- (float)dissolve;

@end

@interface RODraggableImageView(Drawing)

- rotateBy:(NXCoord)ang;
- rotateTo:(NXCoord)ang;
- rotateAbout:(NXPoint*)p;
- rotateAboutCenter;

- translateImagesTo:(NXCoord)x :(NXCoord)y;
- translateImagesBy:(NXCoord)x :(NXCoord)y;

- scaleImagesTo:(NXCoord)x :(NXCoord)y;
- scaleImagesBy:(NXCoord)x :(NXCoord)y;

- rotateImagesBy:(NXCoord)ang;
- rotateImagesTo:(NXCoord)ang;
- rotateImagesAbout:(NXPoint*)p;
- rotateImagesAboutCenter;

@end
