// RODraggableView 26.12.1993

#ifdef __GNUG__
extern "Objective-C" {
#endif

#import <appkit/appkit.h>

#ifdef __GNUG__
}
#endif

#import <ReusableObjects/ROAnimator.h>

#define DDB_ANIM_SPEED 		"RODraggableView.AnimationSpeed"
#define DDB_PIXEL_SPEED 	"RODraggableView.PixelSpeed"
#define DDB_LAUNCH_DIST 	"RODraggableView.LaunchDistance"
#define DDB_WINDOW_MAX 		"RODraggableView.MaxWindows"

@protocol RODraggableViewSource	// informal

- (BOOL)validateDrag:draggableViewSubclass;
	// enables a responding superview (window delegate, app delegate) to veto the drag 
	// if returning NO 
- (NXImage*)dragImage;
	// if a subclass provides a draggingImage NextSteps dragging mechanism will be used
	
@end

@protocol RODraggableViewDestination	// informal

- (BOOL)acceptsDrop:draggableViewSubclass;		// key method
- (BOOL)objectDropped:draggableViewSubclass;	// key method User drop
- performObjectDrop:draggableViewSubclass;		// performDrag Automatic drop
- (NXPoint *)dragToPosition;					// perform the drag to ? position.
- clipRotatedSubview:(id)draggableViewSubclass;	// optional pass of an id just before drawSelf 

@end

@interface RODraggableView:View
{
	ROAnimator*	animator;
	
	struct _dvFlags
	{
		#ifdef __BIG_ENDIAN__
		unsigned int eventsEnabledFlag:1;
		unsigned int followme:1;
		unsigned int moveWhenDragged:1;
		unsigned int imageDrag:1;
		unsigned int removeWhenDragged:1;
		
		unsigned int hide:1;
		
		unsigned int reserved:26;
		#else
		unsigned int reserved:26;
		
		unsigned int hide:1;
		
		unsigned int removeWhenDragged:1;
		unsigned int imageDrag:1;
		unsigned int moveWhenDragged:1;
		unsigned int followme:1;
		unsigned int eventsEnabledFlag:1;
		#endif
	} dvFlags;
	
	Window*	compositeWindow;
	NXPoint dragOrigin,dragCurrent,dragABS,dragDestination;		// screen coordinates !
	float	deltaX,deltaY;
	id		target;
}

+ defaultsChanged:sender;
+ setAnimationSpeed:(float)i;
+ (float)animationSpeed;
+ setMaxWindows:(int)f;
+ (int)maxWindows;
+ setPixelSpeed:(int)f;
+ (int)pixelSpeed;
+ setLaunchWindowDistance:(int)f;
+ (int)launchWindowDistance;

+ (void)sync;
+ (void)syncWithTimeout:(int)seconds;
+ setClipSuperview:(BOOL)b;

- enableDragging:(BOOL)b;
- (BOOL) isDraggingEnabled;

- moveWhenDragged:(BOOL)b;
- (BOOL) doesMoveWhenDragged;

- performDragToView:aView;
- (NXPoint *)mouseLocation;

- slideBack;					// private will crash !
- continueDragToView:aView;		// private will crash !
- (void)step;					// private will crash !

@end