// ROFileSupport.h created by Katzlberger Thomas (cat) - So 01. Jnner 1994    V 1.5 

#ifdef __GNUG__
extern "Objective-C" {
#endif

#import <objc/Object.h>
#include <sys/file.h>
#include <sys/types.h>

#ifdef __GNUG__
}
#endif

#define FILE_SUP [ROFileSupport new]

#define mode_t unsigned short

// modes for file-access ckeck:

#define ROFS_FILE_EXISTS 	F_OK
#define ROFS_READ_OK 		R_OK
#define ROFS_WRITE_OK 		W_OK
#define ROFS_EXECUTE_OK 	X_OK
#define ROFS_CREATE_OK 		0x10
#define ROFS_SAVE_OK 		0x20

// ROFS_CREATE_OK  returns NO if the file exists otherwise it is equal to ROF_SAVE_OK
// ROFS_SAVE_OK    returns YES if the file is writeable OR the file is createable 
//				  (all directory-permissions leading to file are writeable)
//		Simply if it returns YES NXSaveToFile() will succeed if there is enough diskspace

@interface ROFileSupport:Object
{
	char path1[1025];
	char path2[1025];
	char *workPath;
}

+ new;	// returns a shared instance

- init;	// returns a threadsave instance

- (char*)getNXDate;
- (char*)getNXTime;
- (char*)getNXDateAndTime;
- (char*)getLocalTimeWithFormat:(const char*)strftime_format;
- (const char*) userFullName;
- (const char*) userLoginName;

- (char*) appsLibrary:(const char*)inpath;
- (char*) userLibrary:(const char*)inpath;
- (char*) localLibrary:(const char*)inpath;
- (char*) nextLibrary:(const char*)inpath;
- (char*) expandHomeDirectory:(const char*)inpath;

- (char*) getExtensionOf:(const char*)inpath;
- (char*) getWithoutExtensionOf:(const char*)inpath;
- (char*) getNamePartOf:(const char*)inpath;
- (char*) getFilePartOf:(const char*)inpath;
- (char*) getRootPartOf:(const char*)inpath;
- (char*) getPathOf:(const char*)inpath;
- (char*) getConcat:(const char*)dir :(const char*)file;
- (char*) getConcatOf:(const char*)dir with:(const char*)file;

- (char*) replacePathOf:(const char*)inpath with:(const char*)p;
- (char*) replaceNamePartOf:(const char*)inpath with:(const char*)n;
- (char*) replaceFilePartOf:(const char*)inpath with:(const char*)f;
- (char*) replaceExtensionOf:(const char*)inpath with:(const char*)ext;

- (char*) deleteBlanksOf:(char*)inpath;

- (int)  runAlertPanelIfErrno:(const char*)comment;
	// calls perror() and NXRunAlertPanel() if errno!=0

- (BOOL) access:(const char*)file desired:(int)ac;
- (BOOL) symlink:(const char*)linkfile to:(const char*)realfile;
- (const char*) readlink:(const char*)linkfile;
- (BOOL) makeDirectory:(const char*)dir;
- (BOOL) makeDirectory:(const char*)dir withMode:(mode_t)m;
- (BOOL) createPathToFile:(const char*)fullpath;
- (BOOL) createPath:(const char*)dirpath;

- (off_t)  byteSize:(const char*)file;
- (time_t) modificationTime:(const char*)file;
- (time_t) accessTime:(const char*)file;
- (mode_t) fileMode:(const char*)file;

- (BOOL)   isFile:(const char*)f1 newerThan:(const char*)f2;
- (BOOL)   isLink:(const char*)file;
- (BOOL)   isDirectory:(const char*)file;
- (int)unlink:(const char*)file; 			 // == delete a file
- (int)chmod:(const char*)file to:(mode_t)m;

- (NXStream*) newMappedFile;
- (NXStream*) mapFile:(const char*)f;
- (NXStream*) mapFileReadOnly:(const char*)f;
- closeMappedStream:(NXStream*)s;
- closeMappedStream:(NXStream*)s saveTo:(const char*)f;

@end
