// ROStreamEditor.m 11.3.1994

#ifdef __GNUG__
extern "Objective-C" {
#endif

#import <objc/Object.h>
#import <indexing/IXFileFinder.h>
#import <indexing/IXFileRecord.h>

#ifdef __GNUG__
}
#endif

// #import <ReusableObjects/RORegex.h>
#import "RORegex.h"

@protocol ROStreamEditorDelegate	// 1st 2 informal

- streamEditor:sed didFindSelection:(char*)selString;
	// selString is the return val of -getSelection
 
- (BOOL)streamEditor:sed didFindFile:(IXFileRecord *)r;
	// return NO to skip file (or if you want to misuse sed as easy file-finder)
	
- streamEditor:sed didMapFile:(IXFileRecord *)r;
	// apply any of ROStreamEditor's methods now ! (has to be implemented)

@end

@interface ROStreamEditor:Object
{
	char 	  filename[1025];
	
	NXStream* inputStream;
	NXStream* outputStream;

	char	*selectionBuffer,*userSelBuffer;
	int		selBufferLength,inputStreamPos,currentPos,subSelLength,firstLine,lastLine;
	
	struct _sedFlags
	{	
		#ifdef __BIG_ENDIAN__
			unsigned int modCount:16;
			unsigned int uniqueReplace:1;
			unsigned int readonly:1;
			unsigned int lastError:6;
			unsigned int debugMode:1;
			unsigned int selValid:1;
			unsigned int printErrors:1;
			unsigned int returnBOOL:1;
			
			unsigned int reserved:4;
		#else
			unsigned int reserved:4;
			
			unsigned int returnBOOL:1;
			unsigned int printErrors:1;
			unsigned int selValid:1;
			unsigned int debugMode:1;
			unsigned int lastError:6;
			unsigned int readonly:1;
			unsigned int uniqueReplace:1;
			unsigned int modCount:16;
		#endif
	} sedFlags;
	
	RORegex* 		regex;
	IXFileFinder* 	finder;
	
	id 	delegate;
	SEL action;
}

- init;
- initWithSyntax:(unsigned int)s;
- initWithFinderAtPath:(const char*)path;
- initWithFinderInStoreFile:(const char*)fname atPath:(const char*)path;
- setDebug:(BOOL)b;
- createFinderAtPath:(const char*)path;
- (IXFileFinder*)finder;
- (IXPostingList*)performQuery:(const char *)aQuery atPath:(const char *)path;
- setReadOnly:(BOOL)f;
- (BOOL)readOnly;

- setRegexSyntax:(unsigned int)syn;
- (RORegex*)regex;

- setDelegate:d;
- delegate;

- setNotifyAction:(SEL)s;
- setDefaultNotifyAction;

- (BOOL)mapFile:(const char*)file;
- (BOOL)mapFileReadOnly:(const char*)file;
- (BOOL)mergeFile:(const char*)file;
- saveAndClose;
- save;
- (BOOL)saveToFile:(const char*)file;
- close;

- setStream:(NXStream*)s;
- (NXStream*)stream;

- openMemory:(void*)buf size:(int)sz;
- openMemoryReadonly:(const void*)buf size:(int)sz;
- memoryBuffer:(void**)buf size:(int*)sz;
- closeMemory;

- (BOOL)uniquePattern;
- setUniquePattern:(BOOL)f;	
	// if YES the 1st regex match will expand the pattern (this avoids that a pattern 
	// will match two different strings; mainly for safty)

- setFilename:(const char*)f;
- (const char*)filename;
- (int)modificationCount;

- setReturnBOOL:(BOOL)b;
- setPrintErrors:(BOOL)b;

//********************* Query Selection **********************

- (char*)copySelectionFromDefaultZone;
	// the returned buffer will stay effective until YOU free it.
- (char*)getSelection;
	// the buffer will stay effective for 60 sec or the next message sent to this object.

- (int)selectedChars;
- (int)firstSelectedChar;
- (int)lastSelectedChar;

- (int)firstSelectedLine;
- (int)lastSelectedLine;
- (int)selectedLines;

- (int)lineFromPosition:(int)p;
- (int)positionFromLine:(int)l;

//********************* WHOLE TEXT REGEX **********************

- (int)count:(const char*)what;
- (const char*)select:(const char*)what;
- (const char*)selectNext;
- (const char*)selectNext:(const char*)what;
- (const char*)selectNotify:(const char*)what;
- (const char*)subSelect:(const char*)what;
- (const char*)subSelectNext;
- (const char*)subSelectNotify:(const char*)what;

- (const char*)delete:(const char*)what;
- (const char*)insertAfter:(const char*)what with:(const char*)replm;
- (const char*)insert:(const char*)what with:(const char*)replm;
- (const char*)replace:(const char*)what with:(const char*)replm;

//**************** Modifying without Regex ********************

- deleteBeforeSelStart;
- deleteBehindSelStart;
- deleteBehindSelEnd;

@end

