#ifdef __GNUG__
extern "Objective-C" {
#endif

#import <objc/Storage.h>

#ifdef __GNUG__
}
#endif

#define ROSS_STORE_NORMAL		0
#define ROSS_STORE_REFERENCE	1
#define ROSS_STORE_STRING		2
#define ROSS_STORE_OBJECT		3

#define ROSS_SORT_DESCENDING 0
#define ROSS_SORT_ASCENDING  1
#define ROSS_SORT_ASC_FILES	 2		// sort by Filename without path although path exists
#define ROSS_SORT_DESC_FILES 3		// strings must contain at least one '/'
#define ROSS_SORT_DESC_CASE	 4		// case sensitive sort
#define ROSS_SORT_ASC_CASE   5
#define ROSS_SORT_ASC_CASE_FILES  6
#define ROSS_SORT_DESC_CASE_FILES 7

typedef struct PreloadInfo {
	char	*plLibs;
	char	*plDirs;
	char	*plExts;
	BOOL	plFromAppDir, plFromBundleDir;
} PreloadInfo;

@interface ROStringStore : Storage
{
	struct _ssFlags
	{
	#ifdef __BIG_ENDIAN__
        unsigned int isSorted:1;
		unsigned int freeStrings:1;
		unsigned int addSorted:1;
		unsigned int addIfAbsent:1;
		
		unsigned int addNoCopy:1;
		unsigned int hasPreloadInfo:1;
		unsigned int stripPath:1;
		unsigned int stripExtension:1;
		
		unsigned int loadAppDir:1;
		unsigned int loadBundleDir:1;
		unsigned int sortMode:5;
		unsigned int dontSort:1;

		unsigned int disableNoCopy:1;
		unsigned int debugMode:1;
		unsigned int storeMode:4;
		
		unsigned int reserved:10;
	#else
		unsigned int reserved:10;
		
		unsigned int storeMode:4;
		unsigned int debugMode:1;
		unsigned int disableNoCopy:1;

		unsigned int dontSort:1;
		unsigned int sortMode:5;
		unsigned int loadBundleDir:1;
		unsigned int loadAppDir:1;
		
		unsigned int stripExtension:1;
		unsigned int stripPath:1;
		unsigned int hasPreloadInfo:1;
		unsigned int addNoCopy:1;
		
		unsigned int addIfAbsent:1;
		unsigned int addSorted:1;
		unsigned int freeStrings:1;
        unsigned int isSorted:1;
	#endif
   	} ssFlags;
		
	PreloadInfo *preloadInfo;	
}

//	defaults:
//	ssFlags.isSorted 		= YES;
//	ssFlags.freeStrings 	= YES;
//	ssFlags.addSorted	 	= YES;
//	ssFlags.addIfAbsent 	= NO;
//	ssFlags.addNoCopy 		= NO;
//	ssFlags.hasPreloadInfo 	= NO;
//	ssFlags.stripPath 		= NO;
//	ssFlags.stripExtension 	= NO;
//	ssFlags.sortMode 		= ROSS_SORT_ASC_FILES;

- initMode:(int)m;

- initAddSorted:(BOOL)s ifAbsent:(BOOL)a noCopy:(BOOL)c;

- empty;
- freeStrings;
- free;

- setAddSorted:(BOOL)s ifAbsent:(BOOL)a noCopy:(BOOL)c;
- (BOOL)addsSorted;
- (BOOL)addsIfAbsent;
- (BOOL)addsNoCopy;

- setStripPath:(BOOL)p stripExtension:(BOOL)e;
- (BOOL)stripsPath;
- (BOOL)stripsExtension;

- setSortMode:(int)h;
- (int)sortMode;
- sortStrings:sender;	// calles sort
- sort;					// ascending
- sortHow:(int)f;		// Shellsort
- (BOOL)isSorted;

// associative functionality:
- setReference:(int)ref at:(unsigned int)idx;
- (int)referenceAt:(unsigned int)idx;
- addKey:(const char*)k reference:(int)v;
- (int)referenceForStringKey:(const char*)k;

- setStringValue:(const char*)s at:(unsigned int)idx;
- (const char*) stringValueAt:(int)idx;
- addKey:(const char*)k stringValue:(const char*)v;
- (const char*) valueForStringKey:(const char*)k;

- setObject:obj at:(unsigned int)idx;
- objectAt:(unsigned int)idx;
- addKey:(const char*)k object:v;
- objectForStringKey:(const char*)k;

// adding strings with parameters set above:
- addString:(const char *)string;			// all adds through this
- addStrings:(const char *const*)strings;
- addStringStore:(ROStringStore*)stringStoreObject;
- addDelimitedStrings:(const char*)string delimiters:(const char*)dels;
- addStringsFromFile:(const char*)fullPath delimiters:(const char*)del;
- addStringsFromFile:(const char*)fullPath regex:(const char*)reg;
- addStringsFromFile:(const char*)fullPath regex:(const char*)reg subregex:(const char*)sreg;

- addFilename:(const char*)f;
- addFilenames:(const char* const*)f;
- addFilenamesFromDirectory:(const char *)dir withExtension:(const char *)ext;
- addFilenamesFromStandardLibrariesDirectory:(const char*)dir withExtension:(const char*)ext;

- (const char *const*)strings;
- (const char *)stringAt:(unsigned int)at;
- (const char *)stringAt:(unsigned int)at stripPath:(BOOL)sp stripExtension:(BOOL)se;

- (BOOL)stringExists:(const char *)string;
- (int)indexOfString:(const char *)string;	// return < 0 if not existing 

- removeString:(const char *)string;
- removeStrings:(const char *const*)strings;
- removeStringAt:(unsigned int)at;			// all removals through this

- write:(NXTypedStream *)stream;
- read:(NXTypedStream *)stream;

- stopModalAndReturnTagOf:sender;
- stopModalOrderOutAndReturnTagOf:sender;	// metdods to stop a modalsession of a panel
- (int)browser:sender fillMatrix:matrix inColumn:(int)column;

@end

@interface ROStringStore(Preloading)

- (PreloadInfo *)preloadInfo;

- (const char *)preLoadLibraries;
- setPreLoadLibraries:(const char *)libraries;

- (const char *)preLoadDirectories;
- setPreLoadDirectories:(const char *)directories;

- (const char *)preLoadExtensions;
- setPreLoadExtensions:(const char *)extensions;

- (BOOL)preLoadFromAppDir;
- setPreLoadFromAppDir:(BOOL)flag;

- (BOOL)preLoadFromBundleDir;
- setPreLoadFromBundleDir:(BOOL)flag;

- (BOOL)preLoadStripPath;
- setPreLoadStripPath:(BOOL)flag;

- (BOOL)preLoadStripExt;
- setPreLoadStripExt:(BOOL)flag;

- (BOOL)preLoadIfAbsent;
- setPreLoadIfAbsent:(BOOL)flag;

- (BOOL)preLoadSorted;
- setPreLoadSorted:(BOOL)flag;

- addPreLoadLibraryAndDirectoryFilenames;

@end

