// Stack.m

#ifdef __GNUG__
extern "Objective-C" {
#endif

#import <appkit/View.h>
#import <appkit/graphics.h>

#ifdef __GNUG__
}
#endif

#import "Card.h"
#import "Deck.h"
#import "StackProtocols.h"

#define GROW_NOT    	0
#define GROW_DOWN 		1
#define GROW_LEFT 		2
#define GROW_RIGHT 		3
#define GROW_UP 		4
#define GROW_ZICKZACK 	5
#define GROW_ZICKZACKV 	6
#define GROW_CUSTOM 	7

#define DELTA_X 22
#define DELTA_Y 20

#define BONUS_DS_ADD		[[deck list] count] * 10
#define BONUS_DS_REMOVE		8 - [[deck list] count] * 10

#define BONUS_TS_ADD		-33
#define BONUS_TS_REMOVE		12

#define BONUS_BS_ADD		-23
#define BONUS_BS_REMOVE		13

#define BONUS_PS_ADD		-15
#define BONUS_PS_REMOVE		5

#define DDB_STACK_BGC "Stack.BackgroundColor"

@interface Stack:View
{
	id deck,logicEvaluator;
	
	int zick,zack;
	
	float 	dx,dy,deltaX,deltaY;
	NXCoord nextx,nexty;
	NXSize  cardSize;
		
	struct
	{
	#ifdef __BIG_ENDIAN__
		unsigned int droppable:2;
		unsigned int dragable:2;
		unsigned int transparent:2;
		unsigned int multiDrop:2;
		unsigned int grow:6;
		unsigned int dragsOut:8;
		unsigned int zickZack:2;
		unsigned int reserved:8;
	#else
		unsigned int reserved:8;
		unsigned int zickZack:2;
		unsigned int dragsOut:8;
		unsigned int grow:6;
		unsigned int multiDrop:2;
		unsigned int transparent:2;
		unsigned int dragable:2;
		unsigned int droppable:2;
	#endif
	} stackFlags;

	int score;
}

- setLogicEvaluator:obj;

- setMultipleDrop:(BOOL)b;
- setTransparent:(BOOL)b;
- (BOOL) transparent;
- enableDrop:(BOOL)b;
- (BOOL) droppingEnabled;
- enableDrag:(BOOL)b;
- (BOOL) draggingEnabled;

- showFaces:(BOOL)b;

- setScore:(int)s;
- (int)getScore;

- (void) setDeltaY:(float)f;
- (void) setDeltaX:(float)f;
- grow:(int)direction;
- (int) getGrowth;
- setCardSize:(NXSize*)s;

- (Deck*)deck;

- clear;
- reset;

- (BOOL) objectDropped:theObject;

- (Card*)findLowestDraggable;
- (BOOL) acceptsCard:(Card*)card;
- (NXPoint *)dragToPosition;	// for draggable View
- (NXPoint *)nextPosition;		// for Hint System
//
// drawing:
- (void) clipView:view;
- (void) clipRotatedSubview:view;
- drawSelf:(const NXRect *)rects :(int)rectCount;

@end

@interface Stack(ListOps)

- (BOOL)isEmpty;
- (Card*)lastCard;
- (Card*)firstCard;
- (Card*)cardAt:(int)i;
- addCard:(Card*)card;
- addCards:(List*)clist;
- (Card*)replaceCardAt:(int)i with:(Card*)card2;
- (Card*)removeCardAt:(int)i;
- (Card*)removeCard;
- (Card*)removeCard:(Card*)card;
- (List*)removeCards:(Card*)c;		// removes cards down to c
- (List*)selectedCards:(Card*)c;	// adds ids of cards down to c to list

@end

@interface Stack(NeXTDrag)

- (NXDragOperation)draggingEntered:(id <NXDraggingInfo>)info;
- (NXDragOperation)draggingUpdated:(id <NXDraggingInfo>)sender;
- draggingExited:(id <NXDraggingInfo>)sender;
- (BOOL)prepareForDragOperation:(id <NXDraggingInfo>)info;
- (BOOL)performDragOperation:(id <NXDraggingInfo>)info;
- concludeDragOperation:(id <NXDraggingInfo>)sender;

@end
